/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.gui.ShrimpView;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Vector;

import javax.swing.JComboBox;

import ca.uvic.csr.shrimp.DisplayBean.DisplayConstants;
import ca.uvic.csr.shrimp.SelectorBean.SelectorBean;
import ca.uvic.csr.shrimp.gui.ActionManager.CheckBoxAction;


/**
 * Displays the navigation modes used in {@link ShrimpView}.
 * Moved from being an inner class of ShrimpView.
 * 
 * @author Chris Callendar
 * @date 26-Oct-06
 */
public class NavigateModeComboBox extends JComboBox {
	
	private ItemListener il;
	
	public NavigateModeComboBox(Vector modes, SelectorBean selectorBean, 
								final CheckBoxAction magnifyModeAction, 
								final CheckBoxAction fisheyeModeAction, 
								final CheckBoxAction zoomModeAction) {
		super(modes);
        setSelectedItem(selectorBean.getSelected(DisplayConstants.ZOOM_MODE));
        setEditable(false);
        
		il = new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
			    if (e.getStateChange() == ItemEvent.SELECTED) {
			    	String navigateMode = (String) e.getItem();
			    	if (DisplayConstants.FISHEYE.equals(navigateMode))
			    		fisheyeModeAction.startAction();
			    	else if (DisplayConstants.ZOOM.equals(navigateMode))
			    		zoomModeAction.startAction();
			    	else if (DisplayConstants.MAGNIFY.equals(navigateMode))
			    		magnifyModeAction.startAction();
			    } 
			}
		};
		addItemListener(il);
		
		selectorBean.addPropertyChangeListener(DisplayConstants.ZOOM_MODE, new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				removeItemListener(il);
				setSelectedItem(evt.getNewValue());
				addItemListener(il);
			}
		});
	}
}
